
/* 
 * A sample implementation of integer "measurable" lists
 * Implementation by inheritance from integer lists
 */


/* ------------------------------------------------------------ */

object MList {

  def empty ():MList = new MListEmpty()

  def cons (v:Int,l:MList):MList = new MListCons(v,l)
}


/* ------------------------------------------------------------ */

trait MList extends List {

  def isEmpty ():Boolean
  def first ():Int
  def rest ():MList
  def find (f:Int):Boolean

  def append (M:MList):MList

  def length ():Int
  def sum ():Int
}


/* ------------------------------------------------------------ */

trait Common extends MList {

  val accLength:Int
  val accSum:Int

  def downcast (l:List):MList = l match {
    case l2:MList => l2
    case _ => throw new RuntimeException("Illegal downcast to MList")
  }

  def length ():Int = accLength
  def sum ():Int = accSum
}  


/* ------------------------------------------------------------ */

class MListEmpty () extends ListEmpty with Common {

  override def rest ():MList = downcast(super.rest())

  override def append (M:MList):MList = M

  val accLength:Int = 0
  val accSum:Int = 0
}


/* ------------------------------------------------------------ */

class MListCons (v:Int,r:MList) extends ListCons(v,r) with Common {

  override def rest ():MList = downcast(super.rest())

  override def append (M:MList):MList = MList.cons(v,r.append(M))

  val accLength:Int = 1+r.length()
  val accSum:Int = v+r.sum()
}
