
object Rational {

  def apply (numer:Int, denom:Int):Rational = new Rational(numer,denom)
}


class Rational (n:Int,d:Int) extends Ordered[Rational] {

  private def sgn (a:Int):Int = if (a<0) -1 else if (a>0) 1 else 0
  private def abs (a:Int):Int = if (a<0) -a else a
  private def gcd (a:Int,b:Int):Int = if (b==0) a else gcd(b,a % b)

  private val g:Int = gcd(abs(n),abs(d))
  private val numer:Int = (sgn(n*d)) * (abs(n)/g)
  private val denom:Int = (abs(d)/g)

  def unary_- ():Rational = new Rational(-n,d)

  def + (r:Rational):Rational = 
    new Rational(numer*r.denom + r.numer*denom,denom*r.denom)

  def * (r:Rational):Rational = 
    new Rational(numer*r.numer,denom*r.denom)

  def <= (r:Rational):Boolean = (numer*r.denom <= r.numer*denom)


  // CANONICAL METHODS

  override def equals (other:Any):Boolean = other match {
    case that:Rational => (this<=that && that<=this)
    case _ => false
  }

  override def hashCode ():Int = 
    41 * (
      41 + numer.hashCode()
    ) + denom.hashCode()
  
  override def toString ():String = numer + "/" + denom
  
}
